@@ -26,12 +26,14 @@
 
 #include <QAction>
 #include <QComboBox>
+#include <QDateTime>
 #include <QDir>
 #include <QDragEnterEvent>
 #include <QDropEvent>
 #include <QFileDialog>
 #include <QFileInfo>
 #include <QMimeData>
+#include <QMimeDatabase>
 #include <QLabel>
 #include <QHBoxLayout>
 #include <QKeySequence>
@@ -71,12 +73,29 @@
 
 using namespace ksudoku;
 
+static QString openSaveFilterString() {
+	static QString filterString;
+	if (filterString.isEmpty()) {
+		QStringList mimeFilters;
+		QMimeDatabase db;
+		mimeFilters += db.mimeTypeForName(QStringLiteral("application/xml")).filterString();
+		mimeFilters += db.mimeTypeForName(QStringLiteral("text/plain")).filterString();
+		mimeFilters += i18n("All Files (*)");
+		filterString = mimeFilters.join(QStringLiteral(";;"));
+	}
+	return filterString;
+}
+
 void KSudoku::onCompleted(bool isCorrect, QTime required, bool withHelp) {
 	if(!isCorrect) {
 		KMessageBox::information(this, i18n("Sorry, your solution contains mistakes.\n\nEnable \"Show errors\" in the settings to highlight them."));
 		return;
 	}
 
+	m_timer->stop();
+	setTimerDisplay();
+	m_pauseButton->setEnabled(false);
+
 	QString msg;
 	int secs = QTime(0,0).secsTo(required);
 	int mins = secs / 60;
@@ -89,6 +108,13 @@ void KSudoku::onCompleted(bool isCorrect, QTime required, bool withHelp) {
 			msg = i18np("Congratulations! You made it in 1 minute. With some tricks.", "Congratulations! You made it in %1 minutes. With some tricks.", mins);
 		else
 			msg = i18nc("The two parameters are strings like '2 minutes' or '1 second'.", "Congratulations! You made it in %1 and %2. With some tricks.", i18np("1 minute", "%1 minutes", mins), i18np("1 second", "%1 seconds", secs));
+	else if(m_wasPaused)
+		if (mins == 0)
+			msg = i18np("Congratulations! You made it in 1 second. With at least 1 pause.", "Congratulations! You made it in %1 seconds. With at least 1 pause.", secs);
+		else if (secs == 0)
+			msg = i18np("Congratulations! You made it in 1 minute. With at least 1 pause.", "Congratulations! You made it in %1 minutes. With at least 1 pause.", mins);
+		else
+			msg = i18nc("The two parameters are strings like '2 minutes' or '1 second'.", "Congratulations! You made it in %1 and %2. With at least 1 pause.", i18np("1 minute", "%1 minutes", mins), i18np("1 second", "%1 seconds", secs));
 	else
 		if (mins == 0)
 			msg = i18np("Congratulations! You made it in 1 second.", "Congratulations! You made it in %1 seconds.", secs);
@@ -125,10 +151,14 @@ KSudoku::KSudoku()
 {
 	setObjectName( QStringLiteral("ksudoku" ));
 
-    m_gameWidget = nullptr;
+	// Set game has been paused to false
+	m_wasPaused = false;
+
+	m_gameWidget = nullptr;
     m_gameUI = nullptr;
 
     m_gameActions = nullptr;
+	m_timeDisplay = new QLabel;
 
 	// then, setup our actions
 	setupActions();
@@ -175,11 +205,11 @@ void KSudoku::updateShapesList()
 	// TODO clear the list
     GameVariant* variant = nullptr;
 
-	variant = new SudokuGame(i18n("Sudoku Standard (9x9)"), 9, m_gameVariants);
+	variant = new SudokuGame(this, i18n("Sudoku Standard (9x9)"), 9, m_gameVariants);
 	variant->setDescription(i18n("The classic and fashionable game"));
 	variant->setIcon(QStringLiteral("ksudoku-ksudoku_9x9"));
 #ifdef OPENGL_SUPPORT
-	variant = new RoxdokuGame(i18n("Roxdoku 9 (3x3x3)"), 9, m_gameVariants);
+	variant = new RoxdokuGame(this, i18n("Roxdoku 9 (3x3x3)"), 9, m_gameVariants);
 	variant->setDescription(i18n("The Rox 3D Sudoku"));
 	variant->setIcon(QStringLiteral("ksudoku-roxdoku_3x3x3"));
 #endif
@@ -212,23 +242,23 @@ void KSudoku::updateShapesList()
 
 		variantDataPath = variantDir.filePath(variantDataFile);
 
-		variant = new CustomGame(variantName, QUrl::fromLocalFile(variantDataPath), m_gameVariants);
+		variant = new CustomGame(this, variantName, QUrl::fromLocalFile(variantDataPath), m_gameVariants);
 		variant->setDescription(variantDescr);
 		variant->setIcon(variantIcon);
 	}
 
 	// Put variants first and extra sizes last.
-	variant = new SudokuGame(i18n("Sudoku 16x16"), 16, m_gameVariants);
+	variant = new SudokuGame(this, i18n("Sudoku 16x16"), 16, m_gameVariants);
 	variant->setDescription(i18n("Sudoku with 16 symbols"));
 	variant->setIcon(QStringLiteral("ksudoku-ksudoku_16x16"));
-	variant = new SudokuGame(i18n("Sudoku 25x25"), 25, m_gameVariants);
+	variant = new SudokuGame(this, i18n("Sudoku 25x25"), 25, m_gameVariants);
 	variant->setDescription(i18n("Sudoku with 25 symbols"));
 	variant->setIcon(QStringLiteral("ksudoku-ksudoku_25x25"));
 #ifdef OPENGL_SUPPORT
-	variant = new RoxdokuGame(i18n("Roxdoku 16 (4x4x4)"), 16, m_gameVariants);
+	variant = new RoxdokuGame(this, i18n("Roxdoku 16 (4x4x4)"), 16, m_gameVariants);
 	variant->setDescription(i18n("The Rox 3D sudoku with 16 symbols"));
 	variant->setIcon(QStringLiteral("ksudoku-roxdoku_4x4x4"));
-	variant = new RoxdokuGame(i18n("Roxdoku 25 (5x5x5)"), 25, m_gameVariants);
+	variant = new RoxdokuGame(this, i18n("Roxdoku 25 (5x5x5)"), 25, m_gameVariants);
 	variant->setDescription(i18n("The Rox 3D sudoku with 25 symbols"));
 	variant->setIcon(QStringLiteral("ksudoku-roxdoku_5x5x5"));
 #endif
@@ -254,6 +284,12 @@ void KSudoku::startGame(const Game& game) {
 	g.setMessageParent(view->widget());
 
 	wrapper->layout()->addWidget(widget);
+	wrapper->setObjectName(QStringLiteral("BG"));
+	wrapper->setStyleSheet(QStringLiteral("#BG { background-image: url(:/shapes/icon_pause.png);"
+										"background-repeat: no-repeat;"
+										"background-position: center; }"
+										));
+
 	widget->show();
 	widget->setFocus();
 
@@ -351,6 +387,46 @@ void KSudoku::startGame(const Game& game) {
 	}
 }
 
+void KSudoku::gamePause() {
+
+	if(m_pauseButton->isChecked())
+	{
+		m_wasPaused = true;
+		// Disable all other visible game play buttons
+		action(QStringLiteral("game_new"))->setEnabled(false);
+		action(QStringLiteral("game_restart"))->setEnabled(false);
+		action(QStringLiteral("game_print"))->setEnabled(false);
+		action(QStringLiteral("move_hint"))->setEnabled(false);
+		action(QStringLiteral("move_solve"))->setEnabled(false);
+
+		// Hide/Disable the game widgets
+		m_gameUI->widget()->setDisabled(true);
+		m_gameUI->widget()->hide();
+		m_gameUI->valueListWidget()->hide();
+
+		// Capture the current timer value
+		m_gameTime = m_gameUI->game().msecsElapsed();
+		m_timer->stop();
+	}
+	else{
+		// Enable all other visible game play buttons
+		action(QStringLiteral("game_new"))->setEnabled(true);
+		action(QStringLiteral("game_restart"))->setEnabled(true);
+		action(QStringLiteral("game_print"))->setEnabled(currentGame().isValid());
+		action(QStringLiteral("move_hint"))->setEnabled(!currentGame().allValuesSetAndUsable());
+		action(QStringLiteral("move_solve"))->setEnabled(!currentGame().wasFinished());
+
+		// Show/Enable the game widgets
+		m_gameUI->widget()->setEnabled(true);
+		m_gameUI->widget()->show();
+		m_gameUI->valueListWidget()->show();
+
+		// Set the game timer to the previous value
+		m_gameUI->game().setTime(m_gameTime);
+		m_timer->start();
+	}
+}
+
 void KSudoku::endCurrentGame() {
 	m_valueListWidget->hide();
 
@@ -466,6 +542,8 @@ void KSudoku::setupActions()
 	KGameStandardAction::gameNew(this, &KSudoku::gameNew, actionCollection());
 	KGameStandardAction::restart(this, &KSudoku::gameRestart, actionCollection());
 	KGameStandardAction::load(this, &KSudoku::gameOpen, actionCollection());
+	m_pauseButton = KGameStandardAction::pause(this, &KSudoku::gamePause, actionCollection());
+	KActionCollection::setDefaultShortcut(m_pauseButton, QKeySequence(Qt::Key_F8));
 	m_gameSave = KGameStandardAction::save(this, &KSudoku::gameSave, actionCollection());
 	m_gameSaveAs = KGameStandardAction::saveAs(this, &KSudoku::gameSaveAs, actionCollection());
 	KGameStandardAction::print(this, &KSudoku::gamePrint, actionCollection());
@@ -541,6 +619,19 @@ void KSudoku::setupStatusBar (int difficulty, int symmetry)
 	symmetryBox->addItem(i18nc("Symmetry of layout of clues", "Random Choice"));
 	symmetryBox->addItem(i18n("No Symmetry"));
 	symmetryBox->setCurrentIndex (symmetry);
+
+	// Setup the timer display in the status bar
+	QFont font(QStringLiteral("Monospace"));
+	font.setFixedPitch(true);
+	m_timeDisplay->setFrameStyle(QFrame::NoFrame);
+	m_timeDisplay->setFont(font);
+	m_timeDisplay->setText(QStringLiteral("00:00:00"));
+	statusBar()->addWidget (new QLabel (i18nc("@action", "Elapsed Time:")));
+	statusBar()->addWidget (m_timeDisplay);
+	m_timer = new QTimer(this);
+	connect(m_timer, &QTimer::timeout, this, &KSudoku::setTimerDisplay);
+	m_timer->setInterval(500);
+	m_timer->start();
 }
 
 void KSudoku::adaptActions2View() {
@@ -550,6 +641,7 @@ void KSudoku::adaptActions2View() {
 	m_gameSaveAs->setEnabled(game.isValid());
 	action(QStringLiteral("game_new"))->setEnabled(game.isValid());
 	action(QStringLiteral("game_restart"))->setEnabled(game.isValid());
+	action(QStringLiteral("game_pause"))->setEnabled(game.isValid());
 	action(QStringLiteral("game_print"))->setEnabled(game.isValid());
 	if(game.isValid()) {
 		bool isEnterPuzzleMode = !game.puzzle()->hasSolution();
@@ -674,7 +766,6 @@ void KSudoku::gameRestart()
 	if (!currentView()) return;
 
 	auto game = currentGame();
-
 	// only show question when the current game hasn't been finished until now
 	if (!game.wasFinished()) {
 		if (KMessageBox::questionTwoActions(this,
@@ -686,7 +777,11 @@ void KSudoku::gameRestart()
 		}
 	}
 
+	m_wasPaused = false;
+	action(QStringLiteral("game_pause"))->setEnabled(game.isValid());
 	game.restart();
+	game.setTime(0);
+	m_timer->start();
 }
 
 void KSudoku::gameOpen()
@@ -695,7 +790,8 @@ void KSudoku::gameOpen()
 	// the Open shortcut is pressed (usually CTRL+O) or the Open toolbar
 	// button is clicked
 	// standard filedialog
-	const QUrl Url = QFileDialog::getOpenFileUrl(this, QString(), QUrl::fromLocalFile(QDir::homePath()), QString());
+	const QString docsDir = QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation);
+	const QUrl Url = QFileDialog::getOpenFileUrl(this, QString(), QUrl::fromLocalFile(docsDir), openSaveFilterString());
 
 	if (!Url.isEmpty() && Url.isValid())
 	{
@@ -722,10 +818,12 @@ void KSudoku::gameSave()
     // save the current file
 
 	Game game = currentGame();
-	if(!game.isValid()) return;
+	
+	if(game.getUrl().isEmpty()) {
 
-	if(game.getUrl().isEmpty()) game.setUrl(QFileDialog::getSaveFileUrl());
-	if (!game.getUrl().isEmpty() && game.getUrl().isValid())
+		gameSaveAs();
+	}
+	else if (!game.getUrl().isEmpty() && game.getUrl().isValid())
 	{
 		QString errorMsg;
 		if (!ksudoku::Serializer::store(game, game.getUrl(), this, errorMsg))
@@ -737,9 +835,14 @@ void KSudoku::gameSaveAs()
 {
     // this slot is called whenever the Game->Save As menu is selected,
 	Game game = currentGame();
-	if(!game.isValid()) return;
-
-	game.setUrl(QFileDialog::getSaveFileUrl());
+	if(!game.isValid()) 
+		return;
+	const QDir docsDir(QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation));
+	const QString fName = i18n("Game_") +
+		 QDateTime::currentDateTime().toString(QStringLiteral("yyyy_MM_dd_hhmmss")) +
+		QStringLiteral(".ksudoku.xml");
+	const QUrl fileName = QUrl::fromLocalFile(docsDir.filePath(fName));
+	game.setUrl(QFileDialog::getSaveFileUrl(this,i18n("Save Game"),fileName, openSaveFilterString()));
     if (!game.getUrl().isEmpty() && game.getUrl().isValid())
     	gameSave();
 }
@@ -880,6 +983,32 @@ void KSudoku::enableMessages()
 	}
 }
 
+void KSudoku::changeEvent(QEvent *event)
+{
+	// Reimplemented to handle window minimize/maximize events
+	if (event->type() == QEvent::WindowStateChange && !m_pauseButton->isChecked())
+	{
+		if (event->spontaneous() && !isMinimized() && currentGame().isValid())
+		{
+			// On maximize, set the game timer to the previous value
+			m_gameUI->game().setTime(m_gameTime);
+		}
+		else if (event->spontaneous() && isMinimized() && currentGame().isValid())
+		{
+			// On minimize, capture the current game timer value
+			m_gameTime = m_gameUI->game().msecsElapsed();
+		}
+	}
+}
+
+void KSudoku::setTimerDisplay()
+{
+	if(currentGame().isValid())
+	{
+		m_timeDisplay->setText(currentGame().time().toString(QStringLiteral("hh:mm:ss")));
+	}
+}
+
 #if 0
 KSudokuNewStuff::KSudokuNewStuff( KSudoku* v ) :
         KNewStuff( "ksudoku", (QWidget*) v )
